import sys
import shutil
import tempfile
import random

import pytest

from quixote import cleanup
from wcs import publisher
from wcs.qommon.storage import StorableObject

def setup_module(module):
    cleanup()

    global pub

    publisher.WcsPublisher.APP_DIR = tempfile.mkdtemp()
    pub = publisher.WcsPublisher.create_publisher()


def teardown_module(module):
    shutil.rmtree(pub.APP_DIR)


class Test(StorableObject):
    _names = 'tests%s' % random.randint(0, 100000)
    _indexes = ['unique_value']
    _hashed_indexes = ['value', 'dict_value']

    value = None
    unique_value = None
    dict_value = None

def test_store():
    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value'
    test.store()
    assert test.id == 1

def test_get():
    test = Test.get(1)
    assert test.value == 'value'

def test_get_on_index():
    test = Test.get_on_index('unique-value', 'unique_value')
    assert test.value == 'value'

def test_remove_self():
    test = Test.get(1)
    test.remove_self()
    with pytest.raises(KeyError):
        test = Test.get(1)
    test = Test.get(1, ignore_errors=True)
    assert test is None

def test_get_on_index_changes():
    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value'
    test.store()
    test = Test.get_on_index('unique-value', 'unique_value')
    assert test.value == 'value'

    test.unique_value = 'unique-value2'
    test.store()
    test = Test.get_on_index('unique-value2', 'unique_value')
    assert test.value == 'value'

    with pytest.raises(KeyError):
        test = Test.get_on_index('unique-value', 'unique_value')

def test_get_with_indexed_value():
    Test.wipe()

    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value'
    test.store()

    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value2'
    test.store()

    test = Test()
    test.value = 'value1'
    test.unique_value = 'unique-value3'
    test.store()

    tests = Test.get_with_indexed_value('value', 'value')
    assert len(tests) == 2
    assert 'unique-value' in [x.unique_value for x in tests]
    assert 'unique-value2' in [x.unique_value for x in tests]
    assert 'unique-value3' not in [x.unique_value for x in tests]

def test_get_with_indexed_value_changes():
    Test.wipe()

    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value'
    test.store()

    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value2'
    test.store()

    test = Test()
    test.value = 'value1'
    test.unique_value = 'unique-value3'
    test.store()

    tests = Test.get_with_indexed_value('value', 'value')
    assert len(tests) == 2
    assert 'unique-value' in [x.unique_value for x in tests]
    assert 'unique-value2' in [x.unique_value for x in tests]
    assert 'unique-value3' not in [x.unique_value for x in tests]

    test = Test.get_on_index('unique-value3', 'unique_value')
    test.value = 'value'
    test.store()
    tests = Test.get_with_indexed_value('value', 'value')
    assert len(tests) == 3
    assert 'unique-value3' in [x.unique_value for x in tests]

def test_get_with_indexed_value_dict():
    Test.wipe()

    test = Test()
    test.unique_value = 'unique-value'
    test.store()

    test = Test()
    test.value = 'value'
    test.unique_value = 'unique-value2'
    test.dict_value = {'plop': '2'}
    test.store()

    test = Test()
    test.value = 'value1'
    test.unique_value = 'unique-value3'
    test.dict_value = {'plop': '2'}
    test.store()

    tests = Test.get_with_indexed_value('dict_value', '2')
    assert len(tests) == 2
    assert 'unique-value' not in [x.unique_value for x in tests]
    assert 'unique-value2' in [x.unique_value for x in tests]
    assert 'unique-value3' in [x.unique_value for x in tests]


def test_get_with_indexed_value_dict_changes():
    Test.wipe()

    test1 = Test()
    test1.unique_value = 'unique-value'
    test1.store()

    test2 = Test()
    test2.value = 'value'
    test2.unique_value = 'unique-value2'
    test2.dict_value = {'plop': '2'}
    test2.store()

    test3 = Test()
    test3.value = 'value1'
    test3.unique_value = 'unique-value3'
    test3.dict_value = {'plop': '2'}
    test3.store()

    tests = Test.get_with_indexed_value('dict_value', '2')
    assert len(tests) == 2
    assert 'unique-value' not in [x.unique_value for x in tests]
    assert 'unique-value2' in [x.unique_value for x in tests]
    assert 'unique-value3' in [x.unique_value for x in tests]

    test1.dict_value = {'plop': '2'}
    test1.store()

    tests = Test.get_with_indexed_value('dict_value', '2')
    assert len(tests) == 3

    test1.dict_value = None
    test1.store()

    tests = Test.get_with_indexed_value('dict_value', '2')
    assert len(tests) == 2
